<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

error_reporting(E_ALL | E_STRICT);
require_once('../src/Data.php');
require_once('../src/InputObj.php');
require_once('../src/Rasterization.php');
require_once('../src/Renderer.php');
require_once('../src/RendererBackFaceCulling.php');
require_once('../src/RendererLighting.php');
require_once('../src/OutputSvg.php');
require_once('../src/OutputPngRaster.php');
require_once('../src/OutputPngVector.php');

// Create filters.
$useRasterization = false;
if(isset($_REQUEST['format']) && $_REQUEST['format'] === 'svg') {
	$output = new OutputSvg;
} else if(isset($_REQUEST['format']) && $_REQUEST['format'] === 'pngvector') {
	$output = new OutputPngVector;
} else {
	$useRasterization = true;
	$output = new OutputPngRaster;
}
$rasterer = null;
if($useRasterization) {
	$rasterer = new Rasterization;
}
$renderer = new Renderer;
$backFaceCulling = new RendererBackFaceCulling($renderer);
$lighting = new RendererLighting;
$loader = new InputObj;

// Configure filters.
$mvm = new Transformation;
if(isset($_REQUEST['rx'])) {
	$rx = new Transformation;
	$rx->makeRotate(1.0, 0.0, 0.0, floatval($_REQUEST['rx']));
	$mvm = $mvm->mult($rx);
}
if(isset($_REQUEST['ry'])) {
	$ry = new Transformation;
	$ry->makeRotate(0.0, 1.0, 0.0, floatval($_REQUEST['ry']));
	$mvm = $mvm->mult($ry);
}
if(isset($_REQUEST['rz'])) {
	$rz = new Transformation;
	$rz->makeRotate(0.0, 0.0, 1.0, floatval($_REQUEST['rz']));
	$mvm = $mvm->mult($rz);
}
$s = new Transformation;
$s->makeScale(100.0, 100.0, 100.0);
$mvm = $mvm->mult($s);
$renderer->setModelView($mvm);

$proj = new Projection;
$proj->makePerspective(60.0 * M_PI / 180.0, 1.0, 1.0, 100.0);
$renderer->setProjection($proj);

// Add filters to chain.
$chain = array();
$chain[] = $loader;
$chain[] = $backFaceCulling;
$chain[] = $lighting;
$chain[] = $renderer;
if($useRasterization) {
	$chain[] = $rasterer;
}
$chain[] = $output;
$numFilters = count($chain);

// Connect filters.
for($i = 0; $i < $numFilters - 1; $i++) {
	$chain[$i]->attachSink($chain[$i + 1]);
}

// Start streaming.
$chain[0]->receive(new File('Monkey.obj'));

// Generate profiling output.
$sum = 0.0;
$times = array();
$mem = array();
$names = array();
for($i = 0; $i < $numFilters; $i++) {
	$time = $chain[$i]->getProcessTime();
	$sum += $time;
	$times[$i] = $time;
	$mem[$i] = $chain[$i]->getMemoryUsage();
	$names[$i] = get_class($chain[$i]);
}

$profiling = '    Monkey.obj'."\n";
for($i = 0; $i < $numFilters; $i++) {
	$profiling .= "          |\n";
	$profiling .= " ,--------'\n";
	$profiling .= str_pad('(      ', 79, '_')."\n";
	$profiling .= str_pad(' `--->| '.$i.': '.$names[$i], 39, ' ').sprintf("%10.3F MiB   %10.3F ms   %5.1F%% |\n", $mem[$i] / 1024 / 1024, $times[$i] * 1000.0, ($times[$i] / $sum) * 100.0);
	$profiling .= str_pad('      \'', 79, '-')."'\n";
}
$profiling .= "          |\n";
$profiling .= "<---------'\n";
$profiling .= str_repeat(' ', 56).sprintf("%10.3F ms   100.0%%\n", $sum * 1000.0);

file_put_contents('frontend.out', $profiling);
/*
$xAxis = new Face;
$xAxis->vertices[] = new Vector4(0.0, 0.0, 0.0, 1.0);
$xAxis->vertices[] = new Vector4(3.0, 0.0, 0.0, 1.0);
$xAxis->colors[] = new Vector4(1.0, 0.0, 0.0, 0.0);
$xAxis->colors[] = new Vector4(1.0, 0.0, 0.0, 0.0);
$yAxis = new Face;
$yAxis->vertices[] = new Vector4(0.0, 0.0, 0.0, 1.0);
$yAxis->vertices[] = new Vector4(0.0, 3.0, 0.0, 1.0);
$yAxis->colors[] = new Vector4(0.0, 1.0, 0.0, 0.0);
$yAxis->colors[] = new Vector4(0.0, 1.0, 0.0, 0.0);
$zAxis = new Face;
$zAxis->vertices[] = new Vector4(0.0, 0.0, 0.0, 1.0);
$zAxis->vertices[] = new Vector4(0.0, 0.0, 3.0, 1.0);
$zAxis->colors[] = new Vector4(0.0, 0.0, 1.0, 0.0);
$zAxis->colors[] = new Vector4(0.0, 0.0, 1.0, 0.0);
$data->faces[] = $xAxis;
$data->faces[] = $yAxis;
$data->faces[] = $zAxis;
*/
?>
