<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

/**
 * Class storing a path to a file. Used as input to filters.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-05
 */
class File {
	/**
	 * Set the path stored in this object.
	 *
	 * @param $path Path to a file.
	 */
	public function __construct($path) {
		$this->filePath = $path;
	}

	/**
	 * Return the path stored in this object.
	 *
	 * @return Path to a file.
	 */
	public function getPath() {
		return $this->filePath;
	}

	/**
	 * Storage for path.
	 */
	protected $filePath;
}

/**
 * Storage for a material.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-11
 * @see OpenGL function glMaterial
 */
class Material {
	/**
	 * Ambient color.
	 */
	public $ambient;

	/**
	 * Diffuse color.
	 */
	public $diffuse;

	/**
	 * Specular color.
	 */
	public $specular;

	/**
	 * Shininess exponent.
	 */
	public $shininess;
}

/**
 * Storage for a light.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-12
 * @see OpenGL function glLight
 */
class Light {
	/**
	 * Position of the light.
	 */
	public $position;

	/**
	 * Ambient color of the light.
	 */
	public $ambient;

	/**
	 * Diffuse color of the light.
	 */
	public $diffuse;

	/**
	 * Specular color of the light.
	 */
	public $specular;
}



/**
 * Storage for one face.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-11
 */
class Face {
	/**
	 * Array of vertices.
	 */
	public $vertices;

	/**
	 * Array of normals.
	 */
	public $normals;

	/**
	 * Array of texture coordinates.
	 */
	public $texCoords;

	/**
	 * Array of colors.
	 */
	public $colors;

	/**
	 * Material which should be used for rendering this face.
	 */
	public $material;
}

/**
 * Storage class storing scene data. Used for data streaming between filters.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-05
 */
class Data3D {
	/**
	 * Array containing faces (of type @b Face).
	 */
	public $faces;
}

/**
 * Storage class storing polygons in screen space. Used for data streaming 
 * between filters.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-05
 */
class Data2D {
	/**
	 * Array of arrays representing polygons.
	 */
	public $data;
}

/**
 * Storage class for a bitmap of pixels. Used for data streaming between 
 * filters.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2009-01-07
 */
class Bitmap {
	/**
	 * Linear array of pixels in row-major order storing the color data. The 
	 * size of this is @c {$width * $height}. The color components are stored as 
	 * one integer 0xRRGGBBAA. Alpha is not used currently.
	 */
	public $color;

	/**
	 * Linear array of pixels in row-major order storing the depth data. The 
	 * size of this is @c {$width * $height}. The depth is stored as one float.
	 */
	public $depth;

	/**
	 * Width of the image in pixels.
	 */
	public $width;

	/**
	 * Height of the image in pixels.
	 */
	public $height;

}
?>
