<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('MathMatrix.php');

/**
 * Matrix used for projecting from camera space to screen space.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-10-31
 */
class Projection extends Matrix4x4 {
	/**
	 * Default constructor calling parent constructor.
	 */
	function __construct() {
		parent::__construct();
	}

	/**
	 * Create an orthographic projection.
	 *
	 * @param $left Left clipping plane.
	 * @param $right Right clipping plane.
	 * @param $bottom Bottom clipping plane.
	 * @param $top Top clipping plane.
	 * @param $near Near z clipping plane.
	 * @param $far Far z clipping plane.
	 * @see OpenGL function glOrtho
	 */
	function makeOrthographic($left, $right, $bottom, $top, $near, $far) {
		$this->data[ 0] = 2.0 / ($right - $left);
		$this->data[ 1] = 0.0;
		$this->data[ 2] = 0.0;
		$this->data[ 3] = ($right + $left) / ($right - $left);

		$this->data[ 4] = 0.0;
		$this->data[ 5] = 2.0 / ($top - $bottom);
		$this->data[ 6] = 0.0;
		$this->data[ 7] = ($top + $bottom) / ($top - $bottom);

		$this->data[ 8] = 0.0;
		$this->data[ 9] = 0.0;
		$this->data[10] = 2.0 / ($far - $near);
		$this->data[11] = ($far + $near) / ($far - $near);

		$this->data[12] = 0.0;
		$this->data[13] = 0.0;
		$this->data[14] = 0.0;
		$this->data[15] = 1.0;
	}

	/**
	 * Create a perspective projection.
	 *
	 * @param $fovy Field of view in y direction in radians.
	 * @param $aspect Aspect ratio of width to height.
	 * @param $near Near z clipping plane.
	 * @param $far Far z clipping plane.
	 * @see OpenGL Utility function gluPerspective
	 */
	function makePerspective($fovy, $aspect, $near, $far) {
		$f = 1.0 / tan($fovy / 2.0);
		
		$this->data[ 0] = $f / $aspect;
		$this->data[ 1] = 0.0;
		$this->data[ 2] = 0.0;
		$this->data[ 3] = 0.0;

		$this->data[ 4] = 0.0;
		$this->data[ 5] = $f;
		$this->data[ 6] = 0.0;
		$this->data[ 7] = 0.0;

		$this->data[ 8] = 0.0;
		$this->data[ 9] = 0.0;
		$this->data[10] = ($far + $near) / ($near - $far);
		$this->data[11] = (2.0 * $far * $near) / ($near - $far);

		$this->data[12] = 0.0;
		$this->data[13] = 0.0;
		$this->data[14] = -1.0;
		$this->data[15] = 0.0;
	}
}
?>
