<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('MathMatrix.php');

/**
 * Class representing a 4 × 4 transformation matrix.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-04
 */
class Transformation extends Matrix4x4 {
	/**
	 * Default constructor calling parent constructor.
	 */
	function __construct() {
		parent::__construct();
	}

	/**
	 * Create a scaling transformation.
	 *
	 * @param $sx Scaling in x direction.
	 * @param $sy Scaling in y direction.
	 * @param $sz Scaling in z direction.
	 */
	function makeScale($sx, $sy, $sz) {
		$this->makeIdentity();
		$this->data[ 0] = $sx;
		$this->data[ 5] = $sy;
		$this->data[10] = $sz;
	}

	/**
	 * Create a translation transformation.
	 *
	 * @param $dx Translation in x direction.
	 * @param $dy Translation in y direction.
	 * @param $dz Translation in z direction.
	 */
	function makeTranslate($dx, $dy, $dz) {
		$this->makeIdentity();
		$this->data[ 3] = $dx;
		$this->data[ 7] = $dy;
		$this->data[11] = $dz;
	}

	/**
	 * Create a rotation transformation.
	 *
	 * @param $ax Value in x direction of rotation axis.
	 * @param $ay Value in y direction of rotation axis.
	 * @param $az Value in z direction of rotation axis.
	 * @param $angle Rotation angle in radians.
	 */
	function makeRotate($ax, $ay, $az, $angle) {
		$cosA = cos($angle);
		$cosOneA = 1.0 - $cosA;
		$sinA = sin($angle);
		
		$this->data[ 0] = $cosA + $ax * $ax * $cosOneA;
		$this->data[ 1] = $ax * $ay * $cosOneA - $az * $sinA;
		$this->data[ 2] = $ax * $az * $cosOneA + $ay * $ay * $sinA;
		$this->data[ 3] = 0.0;

		$this->data[ 4] = $ax * $ay * $cosOneA + $az * $sinA;
		$this->data[ 5] = $cosA + $ay * $ay * $cosOneA;
		$this->data[ 6] = $ay * $az * $cosOneA - $ax * $sinA;
		$this->data[ 7] = 0.0;

		$this->data[ 8] = $ax * $az * $cosOneA - $ay * $sinA;
		$this->data[ 9] = $ay * $az * $cosOneA + $ax * $sinA;
		$this->data[10] = $cosA + $az * $az * $cosOneA;
		$this->data[11] = 0.0;

		$this->data[12] = 0.0;
		$this->data[13] = 0.0;
		$this->data[14] = 0.0;
		$this->data[15] = 1.0;
	}
}
?>
