<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('Data.php');
require_once('Filter.php');

/**
 * Class for output of raster graphics as Portable Network Graphics (PNG).
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-16
 * @see http://www.libpng.org/pub/png/
 */
class OutputPngRaster extends Filter {

	/**
	 * This class accepts a Bitmap object as input and generates output.
	 */
	public function __construct() {
		parent::__construct('Bitmap', 'Bitmap');
	}

	/**
	 * Do the actual PNG output.
	 *
	 * @param $data Object containing a bitmap
	 */
	public function process(& $data) {
		// Initialize image.
		$image = imagecreatetruecolor($data->width, $data->height);

		// Set background color.
		$bg = imagecolorallocate($image, 255, 255, 255);
		imagefilledrectangle($image, 0, 0, $data->width - 1, $data->height - 1, $bg);
		imagecolordeallocate($image, $bg);

		// Draw image.
		for($y = 0; $y < $data->height; $y++) {
			for($x = 0; $x < $data->width; $x++) {
				$index = $data->width * $y + $x;
				// Skip pixels which have not been set (background).
				if(isset($data->depth[$index])) {
					$pixel = $data->color[$data->width * $y + $x];
					$r = ($pixel & 0xFF000000) >> 24;
					$g = ($pixel & 0x00FF0000) >> 16;
					$b = ($pixel & 0x0000FF00) >> 8;
					$color = imagecolorallocate($image, $r, $g, $b);
					imagesetpixel($image, $x, $y, $color);
					imagecolordeallocate($image, $color);
				}
			}
		}

		// Output.
		header('Content-Type: image/png');
		imagepng($image, NULL, 9, PNG_NO_FILTER);
		imagedestroy($image);

		$this->send($data);
	}
}
?>
