<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('Data.php');
require_once('Filter.php');

/**
 * Class for output of vector graphics as Portable Network Graphics (PNG).
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-16
 * @see http://www.libpng.org/pub/png/
 */
class OutputPngVector extends Filter {

	/**
	 * This class accepts a Data2D object as input and generates output.
	 */
	public function __construct() {
		parent::__construct('Data2D', 'Data2D');
	}

	/**
	 * Do the actual PNG output.
	 *
	 * @param $data Object containing 2D polygon data.
	 */
	public function process(& $data) {
		$width = 800;
		$height = 600;
		$halfWidth = $width / 2.0;
		$halfHeight = $height / 2.0;

		// Initialize image.
		$image = imagecreatetruecolor($width, $height);

		// Set background color.
		$bg = imagecolorallocate($image, 255, 255, 255);
		imagefilledrectangle($image, 0, 0, $width - 1, $height - 1, $bg);
		imagecolordeallocate($image, $bg);

		// Draw polygons.
		foreach($data->data as $polygon) {
			if(!is_array($polygon)) {
				throw new Exception('Invalid polygon.');
			}

			$num = count($polygon);
			if($num == 2) {
				$lineColor = imagecolorallocate($image, $polygon[0][2]->x() * 255.0, $polygon[0][2]->y() * 255.0, $polygon[0][2]->z() * 255.0);
				imageline($image, $polygon[0][0] + $halfWidth, $polygon[0][1] + $halfHeight, $polygon[1][0] + $halfWidth, $polygon[1][1] + $halfHeight, $lineColor);
				imagecolordeallocate($image, $lineColor);
			} else {
				$points = array();
				$color = array(0.0, 0.0, 0.0);
				for($i = 0; $i < $num; $i++) {
					$points[] = $polygon[$i][0]->x() + $halfWidth;
					$points[] = $polygon[$i][0]->y() + $halfHeight;
					$color[0] += $polygon[$i][1]->x();
					$color[1] += $polygon[$i][1]->y();
					$color[2] += $polygon[$i][1]->z();
				}
				$polyColor = imagecolorallocate($image, $color[0] / $num * 255.0, $color[1] / $num * 255.0, $color[2] / $num * 255.0);
				imagefilledpolygon($image, $points, $num, $polyColor);
				imagecolordeallocate($image, $polyColor);
			}
		}

		// Output.
		header('Content-Type: image/png');
		imagepng($image, NULL, 9, PNG_NO_FILTER);
		imagedestroy($image);

		$this->send($data);
	}
}
?>
