<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('Data.php');
require_once('Filter.php');

/**
 * Class for output of Scalable Vector Graphics (SVG).
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-10-29
 * @see http://www.w3.org/Graphics/SVG/
 */
class OutputSvg extends Filter {

	/**
	 * This class accepts a Data2D object as input and generates output.
	 */
	public function __construct() {
		parent::__construct('Data2D', 'Data2D');
	}

	/**
	 * Do the actual SVG output.
	 *
	 * @param $data Object containing 2D polygon data.
	 */
	public function process(& $data) {
		header('Content-Type: image/svg+xml; charset=utf-8');
		echo '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>';
		echo '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">';
		echo '<svg version="1.1" xmlns="http://www.w3.org/2000/svg" width="800" height="600">';
		echo '<g transform="translate(400 300)">';
		foreach($data->data as $polygon) {
			if(!is_array($polygon)) {
				throw new Exception('Invalid polygon.');
			}
			$num = count($polygon);
			if($num == 2) {
				printf('<line x1="%F" y1="%F" x2="%F" y2="%F" stroke="rgb(%u, %u, %u)" />',
						$polygon[0][0]->x(), $polygon[0][0]->y(), $polygon[1][0]->x(), $polygon[1][0]->y(),
						$polygon[0][1]->x() * 255.0, $polygon[0][1]->y() * 255.0, $polygon[0][1]->z() * 255.0);
			} else {
				$svgPoly = '<polygon fill="rgb(%u, %u, %u)" stroke="none" points="';
				$color = array(0.0, 0.0, 0.0);
				
				for($i = 0; $i < $num; $i++) {
					$svgPoly .= $polygon[$i][0]->x().','.$polygon[$i][0]->y().' ';
					$color[0] += $polygon[$i][1]->x();
					$color[1] += $polygon[$i][1]->y();
					$color[2] += $polygon[$i][1]->z();
				}
				$svgPoly .= '" />';
				printf($svgPoly, $color[0] / $num * 255.0, $color[1] / $num * 255.0, $color[2] / $num * 255.0);
			}
		}
		echo '</g>';
		echo '</svg>';

		$this->send($data);
	}
}
?>
