<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('Data.php');
require_once('Filter.php');

/**
 * Class for removing faces which are seen from behind.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-11
 */
class RendererBackFaceCulling extends Filter {

	/**
	 * This class accepts a Data3D object as input and generates a Data3D object
	 * as output.
	 */
	public function __construct(Renderer & $renderer) {
		parent::__construct('Data3D', 'Data3D');
		$this->renderer =& $renderer;
	}

	/**
	 * Remove faces which are back facing from the scene.
	 *
	 * @param $data 3D scene data.
	 */
	public function process(& $data) {
		$mvm = $this->renderer->getModelView();
		$viewDir = new Vector4(0.0, 0.0, 1.0, 0.0);

		foreach($data->faces as $key => $face) {
			$numVertices = count($face->vertices);
			if($numVertices < 3) {
				continue;
			}
			$v1 = $face->vertices[1]->sub($face->vertices[0]);
			$v2 = $face->vertices[2]->sub($face->vertices[0]);
			$n = $mvm->multVector($v1->cross($v2));
			if($n->mult($viewDir) > 0.0) {
				unset($data->faces[$key]);
			}
		}
		$this->send($data);
	}

	/**
	 * Reference to the renderer. Used to retrieve the model-view matrix.
	 */
	protected $renderer;
}
?>
