<?php
/**
 * This file is part of be_w3dr. See the file "be_w3dr/license.txt" for license 
 * information.
 * @package		be_w3dr
 * @author		Benjamin Eikel
 * @version		trunk
 */

require_once('Data.php');
require_once('Filter.php');

/**
 * Class for doing lighting calculations.
 *
 * @package be_w3dr
 * @author Benjamin Eikel
 * @date 2008-11-11
 */
class RendererLighting extends Filter {

	/**
	 * This class accepts a Data3D object as input and generates a Data3D object
	 * as output.
	 */
	public function __construct() {
		parent::__construct('Data3D', 'Data3D');
	}

	/**
	 * Do ligthing calculations for all faces of the scene.
	 *
	 * @param $data 3D scene data.
	 */
	public function process(& $data) {
		$light = new Light;
		$light->position = new Vector4(6.0, 6.0, 1.0, 1.0);
		$light->ambient = new Vector4(1.0, 1.0, 1.0, 1.0);
		$light->diffuse = new Vector4(1.0, 1.0, 1.0, 1.0);
		$light->specular = new Vector4(1.0, 1.0, 1.0, 1.0);
		$eyePos = new Vector4(0.0, 0.0, 0.0, 1.0);

		foreach($data->faces as $face) {
			if($face->material != null) {
				$num = count($face->vertices);
				for($i = 0; $i < $num; $i++) {
					$viewDir = $eyePos->sub($face->vertices[$i]);
					$viewDir->normalize();
					$lightDir = $light->position->sub($face->vertices[$i]);
					$lightDir->normalize();
					$halfVec = $lightDir->add($viewDir);
					$halfVec->normalize();
					
					$face->normals[$i]->normalize();
					$nDotL = $face->normals[$i]->mult($lightDir);
					$diffuseCoeff = 0.0;
					$specularCoeff = 0.0;
					if($nDotL > 0.0) {
						$diffuseCoeff = $nDotL;
						$nDotH = $face->normals[$i]->mult($halfVec);
						if($nDotH > 0.0) {
							$specularCoeff = pow($nDotH, $face->material->shininess);
						}
					}

					$face->colors[$i] = new Vector4(
							$face->material->ambient->x() * $light->ambient->x()
						+	$face->material->diffuse->x() * $light->diffuse->x() * $diffuseCoeff
						+	$face->material->specular->x() * $light->specular->x() * $specularCoeff,

							$face->material->ambient->y() * $light->ambient->y()
						+	$face->material->diffuse->y() * $light->diffuse->y() * $diffuseCoeff
						+	$face->material->specular->y() * $light->specular->y() * $specularCoeff,

							$face->material->ambient->z() * $light->ambient->z()
						+	$face->material->diffuse->z() * $light->diffuse->z() * $diffuseCoeff
						+	$face->material->specular->z() * $light->specular->z() * $specularCoeff,

							$face->material->ambient->w() * $light->ambient->w()
						+	$face->material->diffuse->w() * $light->diffuse->w() * $diffuseCoeff
						+	$face->material->specular->w() * $light->specular->w() * $specularCoeff);
				}
			}
		}
		$this->send($data);
	}
}
?>
